#include "projet.h"

/* ======================================================
 * assign_color
 * =====================================================
 */
void
assign_color (GLfloat * amb_diff_array, float altitude, int colorscheme)
{
  switch (colorscheme) {
  case TERR_COLOR_SUMMER:
    assign_color_summer (amb_diff_array, altitude);
    break;

  case TERR_COLOR_MARS:
    assign_color_mars (amb_diff_array, altitude);
    break;

  case TERR_COLOR_VENUS:
    assign_color_venus (amb_diff_array, altitude);
    break;
    
  case TERR_COLOR_OCEAN:
    assign_color_ocean (amb_diff_array, altitude);
    break;  

  default:
    fprintf(stderr,"Warning: unknown color scheme\n");
    assign_color_summer (amb_diff_array, altitude);
    break;
  }

}

/* ======================================================
 * assign_color_*
 *
 * Description: assigne une couleur  un sommet en fonction de son altitude
 *  cette couleur est en fait un tableau  pass plus tard  OpenGL 
 *  via glMaterialfv (GL_AMBIENT_AND_DIFFUSE)
 * Parametres: tableau de GLfloat  remplir, et altitude du sommet 
 * =====================================================
 */
void
assign_color_summer (GLfloat * amb_diff_array, float altitude)
{
  if (altitude < 0.01) {
    amb_diff_array[TERR_RED] = 0.0;
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.4;
  }

  else if (altitude < 0.4 * terrain->max_height) {
    amb_diff_array[TERR_RED] = 0.05;
    amb_diff_array[TERR_GREEN] = 0.5 * altitude / (0.4 * terrain->max_height);
    amb_diff_array[TERR_BLUE] = 0.05;
  }
  
  else if (altitude < 0.6 * terrain->max_height) {
    amb_diff_array[TERR_RED] = 0.2;
    amb_diff_array[TERR_GREEN] = 0.6 * altitude / (0.4 * terrain->max_height);
    amb_diff_array[TERR_BLUE] = 0.2;
  }

  else if (altitude < 0.85 * terrain->max_height) {
    amb_diff_array[TERR_RED] =
      0.3 + 0.4 * (0.6 * altitude) / (0.4 * terrain->max_height);
    amb_diff_array[TERR_GREEN] =
      0.3 + 0.4 * (0.6 * altitude) / (0.4 * terrain->max_height);
    amb_diff_array[TERR_BLUE] =
      0.3 + 0.4 * (0.6 * altitude) / (0.4 * terrain->max_height);
  }

  else {
    amb_diff_array[TERR_RED] = 1.0;
    amb_diff_array[TERR_GREEN] = 1.0;
    amb_diff_array[TERR_BLUE] = 1.0;
  }

  amb_diff_array[3] = 1.0;
}

void
assign_color_ocean (GLfloat * amb_diff_array, float altitude)
{
   if (altitude < 0.01) {
    amb_diff_array[TERR_RED] = 0.0;
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.03;
  }

  else if (altitude < 0.4 * terrain->max_height) {
    amb_diff_array[TERR_RED] = 0.05 * altitude / (0.4 * terrain->max_height);
    amb_diff_array[TERR_GREEN] = 0.05;
    amb_diff_array[TERR_BLUE] = 0.3 * altitude / (0.4 * terrain->max_height);
  }

  else if (altitude < 0.8 * terrain->max_height) {
    amb_diff_array[TERR_RED] = 0.05;
    amb_diff_array[TERR_GREEN] = 0.15;
    amb_diff_array[TERR_BLUE] = 
      0.2 + 0.2 * (0.6 * altitude) / (0.4 * terrain->max_height);
  }

  else {
    amb_diff_array[TERR_RED] = 0.0;
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.3;
  }

  amb_diff_array[3] = 1.0;
}


void
assign_color_mars (GLfloat * amb_diff_array, float altitude)
{
  if (altitude < 0.01) {
    amb_diff_array[TERR_RED] = 0.1;
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.0;
  }

  else if (altitude < 0.4 * terrain->max_height) {
    amb_diff_array[TERR_RED] = 0.5 * altitude / (0.4 * terrain->max_height);
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.0;
  }

  else if (altitude < 0.8 * terrain->max_height) {
    amb_diff_array[TERR_RED] =
      0.3 + 0.3 * (0.6 * altitude) / (0.4 * terrain->max_height);
    amb_diff_array[TERR_GREEN] = 0.05;
    amb_diff_array[TERR_BLUE] = 0.05;
  }

  else {
    amb_diff_array[TERR_RED] = 0.5;
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.0;
  }

  amb_diff_array[3] = 1.0;
}

void
assign_color_venus (GLfloat * amb_diff_array, float altitude)
{
  if (altitude < 0.01) {
    amb_diff_array[TERR_RED] = 0.1;
    amb_diff_array[TERR_GREEN] = 0.2;
    amb_diff_array[TERR_BLUE] = 0.2;
  }

  else if (altitude < 0.4 * terrain->max_height) {
    amb_diff_array[TERR_RED] = 0.5 * altitude / (0.4 * terrain->max_height);
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.4;
  }

  else if (altitude < 0.8 * terrain->max_height) {
    amb_diff_array[TERR_RED] =
      0.3 + 0.3 * (0.6 * altitude) / (0.4 * terrain->max_height);
    amb_diff_array[TERR_GREEN] = 0.15;
    amb_diff_array[TERR_BLUE] = 0.25;
  }

  else {
    amb_diff_array[TERR_RED] = 0.5;
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.5;
  }

  amb_diff_array[3] = 1.0;
}
