/* Fichier "algo_diamond.c"
 * Algorithme dit du "diamond-square", variante plus 
 * labore du midpoint displacement.
 *
 * Implmentation itrative, comme conseill dans divers documents.
 */
 
 /* --- Principe de l'algorithme --- */
 /* http://www.gameprogrammer.com/fractal.html
  *
  * tant que la taille des carrs est > 0
  *   faire le diamond step pour tous les carrs prsents
  *   faire le square step pour tous les losanges prsents
  *   rduire l'intervalle de generation de nombres aleatoires
  *
  * diamond step = diamond_average + valeur aleatoire
  * square step = square_average + valeur aleatoire
  *
  * cf. descriptions fonctions
  */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "terrain.h"

/* ======================================================
 * diamond_average
 *
 * Description: le centre du losange tant donn, cette fonction 
 * retourne la moyenne de ses 4 sommets
 * Parametres: i, j : coordonnees du centre
 *             sideLength : taille d'un demi cot du carr dans lequel 
 *                          s'inscrit le losange
 * =====================================================
 */
float
diamond_average (TER_terrain_t * terrain, int i, int j, int sideLength)
{
  TER_param_diamsquare_t *param = (TER_param_diamsquare_t *)(terrain->spec_param);
  int wrap = param->wrap;

  if (i == 0) {
    if (wrap)
    return ((TER_get_heightmap_value (terrain, 0, j - sideLength) +
	     TER_get_heightmap_value (terrain, 0, j + sideLength) +
	     TER_get_heightmap_value (terrain, terrain->size-1-sideLength,j) + 
       TER_get_heightmap_value (terrain,sideLength, j)) / 4.0);
    else 
     return ((TER_get_heightmap_value (terrain, 0, j - sideLength) +
	     TER_get_heightmap_value (terrain, 0, j + sideLength) +
        TER_get_heightmap_value (terrain,sideLength, j)) / 4.0);
  }
  
  else if (i == terrain->size - 1) {
    if (wrap)
    return ((TER_get_heightmap_value (terrain, i, j - sideLength) +
	     TER_get_heightmap_value (terrain, i, j + sideLength) +
	     TER_get_heightmap_value (terrain, i - sideLength, j) +
	     TER_get_heightmap_value (terrain, sideLength, j)) / 4.0); 
    else
       return ((TER_get_heightmap_value (terrain, i, j - sideLength) +
	     TER_get_heightmap_value (terrain, i, j + sideLength) +
	     TER_get_heightmap_value (terrain, i - sideLength, j)) / 4.0);     
  }
  
  else if (j == 0) {
    if (wrap)
    return ((TER_get_heightmap_value (terrain, i - sideLength, j) +
	     TER_get_heightmap_value (terrain, i + sideLength, j) +
	     TER_get_heightmap_value (terrain, i, j + sideLength) +
	     TER_get_heightmap_value (terrain, i, terrain->size - 1 - j)) / 4.0);   
    else
      return ((TER_get_heightmap_value (terrain, i - sideLength, j) +
	     TER_get_heightmap_value (terrain, i + sideLength, j) +
	     TER_get_heightmap_value (terrain, i, j + sideLength)) / 4.0); 
    
  }
  
  else if (j == terrain->size - 1) {
    if (wrap)
    return ((TER_get_heightmap_value (terrain, i - sideLength, j) +
	     TER_get_heightmap_value (terrain, i + sideLength, j) +
	     TER_get_heightmap_value (terrain, i, j - sideLength) +
	     TER_get_heightmap_value (terrain, i, sideLength)) / 4.0);
    else
      return ((TER_get_heightmap_value (terrain, i - sideLength, j) +
	     TER_get_heightmap_value (terrain, i + sideLength, j) +
	     TER_get_heightmap_value (terrain, i, j - sideLength)) / 4.0);
  }
  
  else
    return ((TER_get_heightmap_value (terrain, i - sideLength, j) +
	     TER_get_heightmap_value (terrain, i + sideLength, j) +
	     TER_get_heightmap_value (terrain, i, j - sideLength) +
	     TER_get_heightmap_value (terrain, i, j + sideLength)) / 4.0);
}

/* ======================================================
 * square_average
 *
 * Description: le centre du carr tant donn, cette fonction 
 * retourne la moyenne de ses 4 sommets
 * Parametres: i, j: centre du carr
 *             sideLength: taille du demi cot du carr
 * =====================================================
 */
float
square_average (TER_terrain_t * terrain, int i, int j, int sideLength)
{
  return ((TER_get_heightmap_value (terrain, i - sideLength, j - sideLength) +
	   TER_get_heightmap_value (terrain, i - sideLength, j + sideLength) +
	   TER_get_heightmap_value (terrain, i + sideLength, j - sideLength) +
	   TER_get_heightmap_value (terrain, i + sideLength,
				j + sideLength)) / 4.0);
}


/* ======================================================
 * TER_gen_terrain_diamond
 *
 * Description: applique, par une methode iterative, le diamond step 
 *  puis le square step  tous les carrs et diamonds prsents.
 * =====================================================
 */
void
TER_gen_terrain_diamond (TER_terrain_t * terrain)
{
  int i, j;
  int side_length;
  int odd;
  float range_reduction;
  int size = terrain->size - 1;
  float z;
  TER_param_diamsquare_t *param = (TER_param_diamsquare_t *) terrain->spec_param;
  float h = param->h;
  float range = param->factor;
  int wrap = param->wrap;

  /* le taux duquel on reduit l'intervalle du nombre aleatoire 
      chaque iteration  */
  range_reduction = (float) pow (2., -h);

  /* taille du premier demi cot = moiti de la taille du terrain */
  side_length = size >> 1;

  /* met les 4 coins  z=0 (meme si a piori, tout le terrain est  0) */
  TER_set_heightmap_value (terrain, 0, 0, 0);
  TER_set_heightmap_value (terrain, 0, size, 0);
  TER_set_heightmap_value (terrain, size, 0, 0);
  TER_set_heightmap_value (terrain, size, size, 0);

  /* on itere jusqu' ce que la taille du cot soit 0 */
  while (side_length) {

    /* diamond step */
    for (i = side_length; i < size; i += side_length) {
      for (j = side_length; j < size; j += side_length) {
	z =
	  range * randomRange (.5f) + square_average (terrain, i, j,
						      side_length);


	TER_set_heightmap_value (terrain, i, j, z);
	if (z > terrain->max_height) {
	  terrain->max_height = z;
	}
	j += side_length;
      }
      i += side_length;
    }

    /* square step */
    odd = 0; 
    for (i = 0; i < size; i += side_length) {
      odd = (odd == 0);
      for (j = 0; j < size; j += side_length) {
	if ((odd) && !j) {
    /* traitement particulier pour les i pairs et les j nuls : le diamond est 
    inexistant -> on saute, on regle ce pb avec la continuit plus bas */
	  j += side_length;
	}

	z = range *
	  randomRange (.5f) + diamond_average (terrain, i, j, side_length);

	TER_set_heightmap_value (terrain, i, j, z);
	if (z > terrain->max_height)
	  terrain->max_height = z;

	/* continuit du terrain aux bords avec bords opposs */
	if ((i == 0)&& (wrap))
	  TER_set_heightmap_value (terrain, size, j,
			       TER_get_heightmap_value (terrain, i, j));
	if ((j == 0) && (wrap))
	  TER_set_heightmap_value (terrain, i, size,
			       TER_get_heightmap_value (terrain, i, j));

	j += side_length;
      }
    }

    /* rrduit l'intervalle de generation de nombre aleatoire */
    range *= range_reduction;
    
    /* divise la taille du carr par 2 */
    side_length >>= 1;
  }
  
  printf ("DIAMOND: max_height=%f\n", terrain->max_height);
}
