/* Fichier "export_png.c"
 * Exportation du terrain en heightfield au format PNG 
 * (http://www.libpng.org)
 *
 * Ncessaire pour le rendu avec POV
 *
 * Attention, il faut libpng pour compiler ce fichier
 */

#include <stdio.h>
#include <stdlib.h>
#include <png.h> /* libpng */

#include "terrain.h"

/* ======================================================
 * TER_export_png_bw
 *
 * Description: cre une image au format PNG en noir et blanc,
 *  qui represente les altitudes ("heightfield"), et qui pourra
 *  etre interprete par POV.
 * Parametres: pointeur sur le terrain  exporter et nom du fichier
 * Renvoie: 1 si OK, 0 sinon
 * =====================================================
 */
int TER_export_png_bw (TER_terrain_t *terrain, char *filename)
{
  png_byte *pngrow = NULL;
  png_structp png_ptr;
  png_info *info_ptr;
  png_byte mask_lo, onebit;   /* for masking off unused bits from byte */ 

  int xsize, ysize;
  int x, y;
  FILE *fpout;
  unsigned int color;

  xsize = terrain->size;
  ysize = terrain->size;
  
  printf("PNG_export: %s\n",filename);
  
  /* on ouvre le fichier  crire */
  fpout = fopen(filename, "wb");
  if (fpout == NULL) {
    fprintf(stderr,"Trouble opening PNG file %s\n",filename);
    return 0;
  }

  /* initialisations de libpng classiques */
  png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
  if (!png_ptr) {
    fclose(fpout);
    fprintf(stderr,"Error opening PNG file.\n");
    return 0;
  }
     
  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr){
    fclose(fpout);
    fprintf(stderr,"Memory error opening PNG file.\n");
    return 0;
  }
     
  if (setjmp(png_ptr->jmpbuf)) {
    png_destroy_write_struct(&png_ptr, &info_ptr);
    fprintf(stderr, "Had error opening PNG file.\n");
    fclose(fpout);
    return 0;
  }
    
  png_init_io(png_ptr, fpout); 
     
  if (setjmp(png_ptr->jmpbuf)) {
    png_destroy_write_struct(&png_ptr, &info_ptr);
    fprintf(stderr, "Had error opening PNG file.\n");
    fclose(fpout);
    return 0;
  }
  
  /* on choisit le mode noir et blanc */  
  png_set_IHDR(png_ptr, info_ptr, xsize, ysize,
       16, PNG_COLOR_TYPE_GRAY, PNG_INTERLACE_NONE,
       PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);

  png_write_info(png_ptr, info_ptr);
  
  
  if ((pngrow = (png_byte *)malloc(2*xsize*sizeof(png_byte))) == NULL) {
    fclose(fpout);
    fprintf(stderr,"Memory allocation error");
    return 0;
  }

  /* on construit le mask byte */
  onebit = 128;
  mask_lo = 0; 
  for(y = 0; y < (16-8); y++) { 
    mask_lo ^= onebit;
    onebit >>= 1;
  }
 
  for(y = 0; y < ysize; y++) {
    for(x = 0; x < xsize; x++) {
    
     /* on calcule la couleur en niveau de gris : noir = 0, blanc = hauteur_max. */   
      color = (unsigned int) (65535.999 *
          (TER_get_heightmap_value(terrain,x,y)/terrain->max_height));
      pngrow[2*x] = color >> 8;
      pngrow[2*x + 1] = (color % 256) & mask_lo;
    }
    png_write_row(png_ptr, pngrow);
  }
     
  if (setjmp(png_ptr->jmpbuf)) {
    png_destroy_write_struct(&png_ptr, &info_ptr);
    fprintf(stderr,"Error closing file");
    fclose(fpout);
    return 1;
  }
  
  png_write_end(png_ptr, NULL);
  png_write_destroy(png_ptr);
  fclose(fpout);
  
  return 1;
}
